//
// TextControlEntry.cpp
// Implementation for TTextControlEntry class
//

#include <stdio.h>
#include <string>
#include "TextControlEntry.h"

enum {
	PMI_ABOUT = 'ham!'
};

//
// property editors for TextControl controls
//
class TTCAlignmentProperty: public TPickListProperty {
public:
	TTCAlignmentProperty(TComponent *Component, TProperty *prop)
	: TPickListProperty(Component, prop)
	{}
	
	virtual void GetValuePickList(TStringList *list)
	{
		list->MakeEmpty();
		list->AddItem("B_ALIGN_LEFT");
		list->AddItem("B_ALIGN_RIGHT");
		list->AddItem("B_ALIGN_CENTER");
	}
};


//
// TTextControlEntry implementation
//
TComponent *TTextControlEntry::CreateComponent(BView *view, TFormDesigner *designer, BRect rect)
{
	TDesignTextControl *TextControl = new TDesignTextControl(rect, "TextControl", "TextControl", designer);
	TextControl->SetClassName(ClassName());
	
	view->LockLooper();
	view->AddChild(TextControl);
	view->UnlockLooper();
	return TextControl;
}

//
// TDesignTextControl implementation
//
TDesignTextControl::TDesignTextControl(BRect frame, const char *name, const char *caption, TFormDesigner *designer)
:	BTextControl(frame, name, caption, caption, 0)
, TControlComponent(0, designer, ctVisual)
{
	SetComponentName(name);
	SetDTView(this); // ALWAYS do this
	InitPropertyList(); // and this
	InitPropertyValues();
	InitEventList();
	InitEventValues();
	
	// Make this control act right in the designer
	TextView()->MakeEditable(false);
	TextView()->MakeSelectable(false);
}

void TDesignTextControl::MouseDown(BPoint pt)
{
	BTextControl::MouseDown(pt);
	DesignerMouseDown(pt, Bounds(), this);
	Invalidate();
}

void TDesignTextControl::Draw(BRect updateRect)
{
	BTextControl::Draw(updateRect);
	DesignerDrawSelection();
}

bool TDesignTextControl::AddComponentContextMenuItems(BPopUpMenu *menu)
{
	return true;
}

bool TDesignTextControl::HandleComponentContextMessage(BMessage *message)
{
	switch(message->what)
	{
		default: 
			return false;
	}
}

bool TDesignTextControl::DefaultEditAction(BPoint pt)
{
	return false;
}

const char *TDesignTextControl::GetPropertyValue(const char *propName) const
{
	string s = propName;
	
	if(s == "Text")
	{
		return Text();
	}
	
	return TControlComponent::GetPropertyValue(propName);
}

void TDesignTextControl::SetPropertyValue(const char *propName, const char *value)
{
	printf("TDesignTextControl::SetPropertyValue('%s', '%s')\n", propName, value);
	if((!propName) || (!value)) return;
	
	string s = propName;
	string v = value;
	
	if(s == "Text")
	{
		if(DTView()->LockLooper())
		{
			SetText(value);
			DTView()->UnlockLooper();
		}
		StoreProperty(propName, value);
		return;
	}
	
	if((s == "LabelAlignment") || (s == "TextAlignment"))
	{
		alignment aeText, aeLabel, an = B_ALIGN_LEFT;
		GetAlignment(&aeLabel, &aeText);
		
		if(v == "B_ALIGN_RIGHT")
			an = B_ALIGN_RIGHT;
		else if(v == "B_ALIGN_CENTER")
			an = B_ALIGN_CENTER;
		
		if(s == "TextAlignment") 
			aeText = an;
		else
			aeLabel = an;
		
		if(DTView()->LockLooper())
		{
			SetAlignment(aeLabel, aeText);
			DTView()->UnlockLooper();
		}
		
		StoreProperty(propName, value);
		return;
	}
	
	if(s == "Divider")
	{
		if(DTView()->LockLooper())
		{
			float div = atof(value);
			SetDivider(div);
			DTView()->UnlockLooper();
		}
		StoreProperty(propName, value);
		return;
	}
	
	TControlComponent::SetPropertyValue(propName, value);
}

void TDesignTextControl::InitPropertyList(void)
{
	TControlComponent::InitPropertyList();
	
	AddPropAndEditor(
		new TProperty(this, "Text", "Text")
	, new TStringProperty(this, 0)
	);
	
	AddPropAndEditor(	
		new TProperty(this, "LabelAlignment", "B_ALIGN_LEFT")
	, new TTCAlignmentProperty(this, 0)	
	);
	
	AddPropAndEditor(	
		new TProperty(this, "TextAlignment", "B_ALIGN_LEFT")
	, new TTCAlignmentProperty(this, 0)	
	);
	
	AddPropAndEditor(	
		new TProperty(this, "Divider", "80")
	, new TIntProperty(this, 0)	
	);
	
	FindProperty("Text")->SetQuoted(true);
}

void TDesignTextControl::InitEventList(void)
{
	TControlComponent::InitEventList();
}

void TDesignTextControl::InitPropertyValues(void)
{
	TControlComponent::InitPropertyValues();
}

void TDesignTextControl::InitEventValues(void)
{
	TControlComponent::InitEventValues();
}

void TDesignTextControl::MessageReceived(BMessage *message)
{
	if(!HandleComponentDesignMessage(message))
	{
		BTextControl::MessageReceived(message);
	}
}

void TDesignTextControl::OnSetConstructorParams(BString &s)
{
	TControlComponent::OnSetConstructorParams(s);	
	
	char buf[256] = "";
	sprintf(buf, "\"%s\"", FindProperty("Text")->Value());
	s += buf;
}


// eof
