//
// Debugging.cpp
// Debugging class and functions
//

#include <Debugging.h>

TDebug::TDebug(TDebugOutputType type, const char *filename)
: m_OutputType(type),
  m_Filename(filename)
{}

// Takes a printf-style argument
void TDebug::Print(const char *fmt, ...)
{
	va_list ap;
	int count;
	char buf[1024];
	
	va_start(ap, &count);
	vsnprintf(buf, sizeof(buf), fmt, ap);
	va_end(ap);
	
	// call the regular-string version of this function
	// to do the output.
	Print(buf);
}

// Takes a regular string argument
void TDebug::Print(const char *s)
{
  switch(m_OutputType)
  {
    case doStdOut:
    {
      _printStdOut(s);
      break;
    }

    case doFile:
    {
      _printToFile(m_Filename, s);
      break;
    }

    case doBoth:
    {
      _printStdOut(s);
      _printToFile(m_Filename, s);
      break;
    }
		
		default: 
		{
			// do nothing
			break;
		}
  }
}

void TDebug::_printToFile(const char *sFilename, const char *s)
{
  FILE *f = fopen(sFilename, "w+");
  if(f)
  {
    fwrite(f, s);
    fclose(f);
  }
}

void TDebug::_printStdOut(const char *s)
{
  printf(s);
}
// eof

